library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/ETCCDI_era5/")
OUTPUTS <- "../OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx5day","TNn","TXx","CDD","WSDI")
limits <- list(c(0,130),c(-15,15),c(5,40),c(20,120),c(0,15))
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))
mask <- loadGridData("../mask_interp.nc",var="tasmax")

##############---spatial plots---##############################################

#Load data
for(ind in 1:length(ETCCDI_index)){
  message(ETCCDI_index[ind])
  files <- list.files(pattern = paste(ETCCDI_index[ind]))
  files_aemet <- list.files(pattern = paste(ETCCDI_index[ind]),path = "../ETCCDI_aemet-5km/")
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di))
    
    if(ETCCDI_index[ind]%in%c("TXn")){
      di[[names(di)]][["Units"]] <- "K"
      nc <- gridArithmetics(nc,273,operator = "-")
    }
    if(ETCCDI_index[ind]%in%c("TXx")){
      di[[names(di)]][["Units"]] <- "K"
      #nc <- gridArithmetics(nc,273,operator = "-")
    }
    
    if(di[[names(di)]][["Units"]]=="m s-1"){
      di[[names(di)]][["Units"]] <- "mm"
      nc <- gridArithmetics(nc,86400000,operator="/")
      palette=brewer.pal(n=11,name="YlGnBu")
    }
    
    if(di[[names(di)]][["Units"]]=="K"){
      palette=rev(brewer.pal(n=11,name="RdYlBu"))
      #nc <- gridArithmetics(nc,273,operator = "-")
      di[[names(di)]][["Units"]] <- "ºC"
    }
    
    if(di[[names(di)]][["Units"]]=="percentage"){
      palette=rev(brewer.pal(n=11,name="RdYlBu"))
    }
    
    if(di[[names(di)]][["Units"]]=="days"){
      palette=brewer.pal(n=11,name="PuBuGn")
    }
    
    if(di[[names(di)]][["Units"]]=="mm"){
      palette=brewer.pal(n=9,name="YlGnBu")
    }
    
    ###############
    #Aggregation of annual/seasonal indices (climatology)
    data <- list()
    
    if(length(nc$Dates$start)<=72){
      index2 <- redim(climatology(nc),drop=T)
      mask2 <- intersectGrid(index2,mask,type="spatial",which.return = 2)
      index2 <- gridArithmetics(index2,mask2,operator="*")
      
      ############
      data[[1]] <- data.frame(Index=as.numeric(index2$Data),
                              Longitude=rep(nc$xyCoords$x,each=length(nc$xyCoords$y)),
                              Latitude=rep(nc$xyCoords$y,length(nc$xyCoords$x)),
                              Season=rep("Annual",length(as.numeric(index2$Data))))
    }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        index2 <- redim(climatology(subsetGrid(nc,season = seasons[[j]])),drop=T)
        mask2 <- intersectGrid(index2,mask,type="spatial",which.return = 2)
        index2 <- gridArithmetics(index2,mask2,operator="*")
        data[[j+1]] <- data.frame(Index=as.numeric(index2$Data),
                                  Longitude=rep(nc$xyCoords$x,each=length(nc$xyCoords$y)),
                                  Latitude=rep(nc$xyCoords$y,length(nc$xyCoords$x)),
                                  Season=rep(names(seasons)[j],length(as.numeric(index2$Data))))
      }}
  }
  
  #Organize data for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  lims <- limits[[ind]]
  datos$Index[which(datos$Index<lims[1])] <- lims[1]
  datos$Index[which(datos$Index>lims[2])] <- lims[2]
  
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_era5_SpatialPattern_IberianPeninsula_lims.png",sep=""),res=300,width=3000,height=1500)
  print(ggplot(data=datos,aes(x=Longitude,y=Latitude,fill=Index)) + geom_tile() + facet_wrap(~Season,nrow = 1) +
          borders("world",colour = "black",size=0.2) + scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
          coord_quickmap(xlim = range(datos$Longitude),ylim=range(datos$Latitude),expand = F) + 
          scale_color_gradientn(di[[names(di)]][["Units"]],colours = palette,
                                aesthetics = "fill",na.value = "white",limits=lims) +
          theme_bw() + ggtitle(paste("ERA5",names(di),sep=" ")) )
  dev.off()
  
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_era5_SpatialPattern_Catalunya_lims.png",sep=""),res=300,width=3000,height=1500)
  print(ggplot(data=subset(datos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),
               aes(x=Longitude,y=Latitude,fill=Index)) + geom_tile() + facet_wrap(~Season,nrow = 1) +
          borders("world",colour = "black",size=0.2) + scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
          coord_quickmap(ylim = c(40,43),xlim=c(0,4),expand = F) + 
          scale_color_gradientn(di[[names(di)]][["Units"]],colours = palette,
                                aesthetics = "fill",na.value = "white",limits=lims) +
          theme_bw() + ggtitle(paste("ERA5",names(di),sep=" ")))
  dev.off()
}
