library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)

rm(list=ls()) ; gc()

setwd("/esarchive/scratch/molmo/")

#Useful functions
sumzero <- function(x){
  length(x[which(x<1)])
}

#Thresholds
spatial.extension = 0.6 #percentile of daily spatial extension with no rain
spell.length = 0.8 #percentile of dry spells lengths

#Load netcdf
nc <- nc_open("prlr-5km_5021_mm_Cat.nc")
nc_var <- ncvar_get(nc,"prlr")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")

var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])
rm(nc_var) ; gc()

#Remove cells with no information
na.points <- rowMeans(var)

try <- apply(X=var[-which(is.na(na.points)),],MARGIN=2,FUN=sumzero)
q80 <- quantile(try,probs=spatial.extension,na.rm=T)

vector_dates <- rep(0,dim(var)[2])
vector_dates[which(try>=q80)] <- 1

names(vector_dates) <- c(1:dim(var)[2])
runs <- rle(vector_dates)

endingDates <- names(runs$values[runs$values==1 & runs$lengths>1]) 
offset <- runs$lengths[which(names(runs$values) %in% endingDates)]
startingDates <- names(vector_dates)[which(names(vector_dates) %in% endingDates) - offset]

lengths <- runs$lengths[runs$values==1 & runs$lengths>1]
threshold <- quantile(lengths,probs=spell.length,na.rm=T)

#New rle
endingDates <- names(runs$values[runs$values==1 & runs$lengths>threshold]) 
offset <- runs$lengths[which(names(runs$values) %in% endingDates)]
startingDates <- names(vector_dates)[which(names(vector_dates) %in% endingDates) - offset]

startingDates <- as.numeric(startingDates)
endingDates <- as.numeric(endingDates)
dates <- list()
for(i in 1:length(startingDates)){
  dates[[i]] <- seq(startingDates[i],endingDates[i],by=1)
}

#Identifico las fechas
vector_dates <- rep(0,length(vector_dates))
vector_dates[unlist(dates)] <- 1

dates <- seq(from=as.Date("1951-01-1"),to=as.Date("2021-12-31"),by="day")
dias_secos <- dates[which(vector_dates==1)]
save(dias_secos,file="fechitas_secas_SpellsThresholds.Rdata")

#Ploteos
#En otro script
